@@ -5,8 +5,8 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 1.5
-;; Package-Requires: ((emacs "27.1") (compat "29.1.4.4"))
+;; Version: 1.6
+;; Package-Requires: ((emacs "27.1") (compat "30"))
 ;; Homepage: https://github.com/minad/cape
 ;; Keywords: abbrev, convenience, matching, completion, text
 
@@ -354,7 +354,7 @@ string as first argument to the completion table."
 
 (declare-function ring-elements "ring")
 (declare-function eshell-bol "eshell")
-(declare-function comint-bol "comint")
+(declare-function comint-line-beginning-position "comint")
 (defvar eshell-history-ring)
 (defvar comint-input-ring)
 
@@ -375,10 +375,12 @@ See also `consult-history' for a more flexible variant based on
       (cond
        ((derived-mode-p 'eshell-mode)
         (setq history eshell-history-ring
-              bol (save-excursion (eshell-bol) (point))))
+              bol (static-if (< emacs-major-version 30)
+                      (save-excursion (eshell-bol) (point))
+                    (line-beginning-position))))
        ((derived-mode-p 'comint-mode)
         (setq history comint-input-ring
-              bol (save-excursion (comint-bol) (point))))
+              bol (comint-line-beginning-position)))
        ((and (minibufferp) (not (eq minibuffer-history-variable t)))
         (setq history (symbol-value minibuffer-history-variable)
               bol (line-beginning-position))))
@@ -814,14 +816,15 @@ If INTERACTIVE is nil the function acts like a Capf."
 ;;;###autoload
 (defun cape-company-to-capf (backend &optional valid)
   "Convert Company BACKEND function to Capf.
-VALID is a function taking the old and new input string.  It
-should return nil if the cached candidates became invalid.  The
-default value for VALID is `string-prefix-p' such that the
-candidates are only fetched again if the input prefix
-changed.  The function `cape-company-to-capf' is experimental."
+VALID is a function taking the old and new input string.  It should
+return nil if the cached candidates became invalid.  The default value
+for VALID is `string-prefix-p' such that the candidates are only fetched
+again if the input prefix changed."
   (lambda ()
     (when (and (symbolp backend) (not (fboundp backend)))
       (ignore-errors (require backend nil t)))
+    (when (bound-and-true-p company-mode)
+      (error "`cape-company-to-capf' should not be used with `company-mode', use the Company backend directly instead"))
     (when (and (symbolp backend) (not (alist-get backend cape--company-init)))
       (funcall backend 'init)
       (put backend 'company-init t)
@@ -905,10 +908,7 @@ multiple super Capfs in the `completion-at-point-functions':
         (list (cape-capf-super \\='eglot-completion-at-point
                                :with \\='tempel-complete)
               (cape-capf-super \\='cape-dabbrev
-                               :with \\='tempel-complete)))
-
-The functions `cape-wrap-super' and `cape-capf-super' are
-experimental."
+                               :with \\='tempel-complete)))"
   (when-let ((results (cl-loop for capf in capfs until (eq capf :with)
                                for res = (funcall capf)
                                if res collect (cons t res))))
@@ -1008,13 +1008,14 @@ experimental."
         ,@(and (not exclusive) '(:exclusive no))
         ,@(mapcan
            (lambda (prop)
-             (list prop (lambda (cand &rest args)
-                          (let ((ref (get-text-property 0 'cape-capf-super cand)))
-                            (when-let ((fun (plist-get
-                                             (or (cdr ref)
-                                                 (and cand-ht (gethash cand cand-ht)))
-                                             prop)))
-                              (apply fun (or (car ref) cand) args))))))
+             (list prop
+                   (lambda (cand &rest args)
+                     (if-let ((ref (get-text-property 0 'cape-capf-super cand)))
+                         (when-let ((fun (plist-get (cdr ref) prop)))
+                           (apply fun (car ref) args))
+                       (when-let ((plist (and cand-ht (gethash cand cand-ht)))
+                                  (fun (plist-get plist prop)))
+                         (apply fun cand args))))))
            cand-functions)))))
 
 ;;;###autoload
@@ -1271,5 +1272,29 @@ This function can be used as an advice around an existing Capf."
       (format "Create a %s Capf from CAPF.
 The Capf calls `%s' with CAPF and ARGS as arguments." name wrapper))))
 
+(defvar-keymap cape-prefix-map
+  :doc "Keymap used as completion entry point.
+The keymap should be installed globally under a prefix."
+  "p" #'completion-at-point
+  "t" #'complete-tag
+  "d" #'cape-dabbrev
+  "h" #'cape-history
+  "f" #'cape-file
+  "s" #'cape-elisp-symbol
+  "e" #'cape-elisp-block
+  "a" #'cape-abbrev
+  "l" #'cape-line
+  "w" #'cape-dict
+  "k"  'cape-keyword
+  ":"  'cape-emoji
+  "\\" 'cape-tex
+  "_"  'cape-tex
+  "^"  'cape-tex
+  "&"  'cape-sgml
+  "r"  'cape-rfc1345)
+
+;;;###autoload (autoload 'cape-prefix-map "cape" nil t 'keymap)
+(defalias 'cape-prefix-map cape-prefix-map)
+
 (provide 'cape)
 ;;; cape.el ends here
